---------------------------------------------------------------
--                                                           
--  ROBOT INTERPRETER
--  By: Martin Carlisle and Keith Shomper     
--                                                           
--  PC_Dialog 
--  Description : procedures for dialog to change Program Counter
--                       
--  By: Martin Carlisle
--                                         
-- ROBOT INTERPRETER is free software; you can redistribute it and/or 
-- modify it without restriction.  However, we ask that you
-- please retain the original author information, and clearly  
-- indicate if it has been modified.
--  
-- ROBOT INTERPRETER is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
--
---------------------------------------------------------------

with RobotADT;
with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;
with State;
with Simulation_Gui;
with Get_PC_Window;
with Instruction_Dialog;
with Mcc.Gui.Widget.Text_Entry;
with Mcc.Gui.Container.Window;
with Mcc.Gui.Keys;
with Controls;
package body PC_Dialog is

   --------
   -- Ok --
   --------
   procedure Ok is
      Good_String : Boolean := True;
   begin
      Get_PC_Window.Read_Window(Good_String);
      if Length(PC_String)=5 then
         declare
            Value : String(1..8) := "2#00000#";
         begin
            Value(3..7) := To_String(PC_String);
            for I in 3..7 loop
               if Value(I)/='0' and Value(I)/='1' then
                  Good_String := False;
                  exit;
               end if;
            end loop;
         end;
      else
         Good_String := False;
      end if;
      if not Good_String then
         Mcc.Gui.Bell;
         Mcc.Gui.Widget.Text_Entry.Highlight(
            Get_PC_Window.PC_Text);
      else
         RobotADT.set_PC(
            R     => State.Robot,
            state => Instruction_Dialog.To_Number(
               To_String(PC_String)));
         Simulation_Gui.Draw_Robot;
         Mcc.Gui.Container.Window.Destroy(
            Get_PC_Window.Get_PC_Window);
      end if;
   end Ok;
   
   procedure Ok (Obj : in out Mcc.Gui.Widget.Button.Button'Class) is
   begin
      Ok;
   end Ok;


   procedure Key_Callback (
      Obj   : in out Mcc.Gui.Sized_Object'Class;
      Event : in     Mcc.Gui.Key_Event) is
      use Mcc.Gui;
   begin
      if Event.Action=Release and Event.Key=Mcc.Gui.Keys.Enter then
         Ok;
      elsif Event.Action=Release and Mcc.Gui.Widget.Text_Entry.Get_Text_Length(
         Get_PC_Window.PC_Text) > 5 then
         Mcc.Gui.Bell;
         declare
            Current : String := Mcc.Gui.Widget.Text_Entry.Get_Text(
               Get_PC_Window.PC_Text);
         begin
            Mcc.Gui.Widget.Text_Entry.Set_Text(
               Obj => Get_PC_Window.PC_Text,
               Text => Current(Current'First..Current'First+4));
         end;
      end if;
   end Key_Callback;


   --------------
   -- PC_Click --
   --------------

   procedure PC_Click
     (Obj   : in out Mcc.Gui.Sized_Object'Class;
      Event : in     Mcc.Gui.Mouse_Event)
   is
      use Mcc.Gui;
      PC : Natural;
      Window_X : Integer := 250;
      Window_Y : Integer := 250;
   begin
      if State.Dialog_Displayed then
         return;
      end if;
      if Event.Button=Left and
         Event.Action=Release  then
         Controls.Stop;
         RobotADT.Get_PC(State.Robot,PC);
         PC_String := To_Unbounded_String(
            Simulation_Gui.Five_Bit_Binary(PC));
         Get_PC_Window.Generate_And_Fill_Window;
         Mcc.Gui.Container.Window.Move(
            Obj => Get_PC_Window.Get_PC_Window,
            X   => Window_X,
            Y   => Window_Y);
         Mcc.Gui.Widget.Text_Entry.Highlight(
            Get_PC_Window.PC_Text);
         Mcc.Gui.Set_Key_Listener(
            Obj      => Get_PC_Window.PC_Text'access,
            Listener => Key_Callback'access);
      end if;
   end PC_Click;

end PC_Dialog;

