---------------------------------------------------------------
--                                                           
--  ROBOT INTERPRETER
--  By: Martin Carlisle and Keith Shomper     
--                                                           
--  SCENE_MENU 
--  Description : Handle choices from Scene menu 
--                       
--  By: Martin Carlisle
--                                         
-- ROBOT INTERPRETER is free software; you can redistribute it and/or 
-- modify it without restriction.  However, we ask that you
-- please retain the original author information, and clearly  
-- indicate if it has been modified.
--  
-- ROBOT INTERPRETER is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
--
---------------------------------------------------------------
with Limits;
with Mcc.Common_Dialogs;
with Ada.Exceptions;
with Room_Size_Window;
with Mcc.Gui.Widget.Text_Entry;
with Mcc.Gui.Container.Window;
with RoomADT;
with State;
with Room_Dialog;
with Mcc.Gui.Event_Scheduler;
with Simulation_Gui;
with RobotADT;

package body Scene_Menu is
   -- used in File Open/Save dialogs to select
   -- which files are displayed
   File_Types : constant String :=
      "{ ""Robot Scene Files"" { .rsc } } " &
      "{ ""All Files"" * }";

   -- Holds file name and Directory name returned from
   -- File dialogs
   Filename       : String(1..Limits.Max_Name_Length);
   Filename_Last  : Natural;
   Directory      : String(1..Limits.Max_Name_Length);
   Directory_Last : Natural;

   -----------------------------
   -- Change_Room_Size_Choice --
   -----------------------------

   procedure Change_Room_Size_Choice is
   begin
      if not Mcc.Gui.Container.Window.Is_Open(
         Room_Size_Window.Room_Size_Window) then
         Room_Size_Window.Generate_and_Fill_Window;
         Mcc.Gui.Widget.Text_Entry.Highlight(
            Room_Size_Window.Room_Size_Text);
         Mcc.Gui.Set_Key_Listener(
            Obj      => Room_Size_Window.Room_Size_Window'access,
            Listener => Room_Dialog.Key_Callback'access);
      end if;
   end Change_Room_Size_Choice;

   -----------------------
   -- Load_Scene_Choice --
   -----------------------

   procedure Load_Scene_Choice is
      I, J : Natural;
   begin
      State.Set_Dialog_Displayed;
      Mcc.Common_Dialogs.Open_Dialog(
         File_Types => File_Types,
         Filename   => Filename,
         File_Last  => Filename_Last,
         Directory  => Directory,
         Dir_Last   => Directory_Last);
      Mcc.Gui.Event_Scheduler.Schedule_Event(500,State.Clear_Dialog_Flag'access);
      if Filename_Last > 0 then
         -- load this window
         RoomADT.Load_Room(
            R        => State.Room, 
            Filename => Filename(Filename'First..Filename_Last));
            
         RobotADT.Get_Location(State.Robot,I,J);
         if RoomADT.Size(State.Room) <= I or RoomADT.Size(State.Room) <= J then
            RobotADT.Set_Location(State.Robot, 1, 1, State.Room'access);
         end if;
         Simulation_Gui.Draw_Room;
      end if;
   exception
      when e:others =>
         Mcc.Common_Dialogs.OK_Box("Unable to open: " &
            Filename(1..Filename_Last) & ASCII.LF &
            "Not a valid Robot Scene file?" & ASCII.LF &
            Ada.Exceptions.Exception_Information(E));
   end Load_Scene_Choice;

   -----------------------
   -- Save_Scene_Choice --
   -----------------------

   procedure Save_Scene_Choice is
   begin
      State.Set_Dialog_Displayed;
      Mcc.Common_Dialogs.Save_Dialog(
         File_Types => File_Types,
         Filename   => Filename,
         File_Last  => Filename_Last,
         Directory  => Directory,
         Dir_Last   => Directory_Last);
      Mcc.Gui.Event_Scheduler.Schedule_Event(500,State.Clear_Dialog_Flag'access);
      if Filename_Last > 0 then
         RoomADT.Store_Room(
            R        => State.Room, 
            Filename => Filename(Filename'First..Filename_Last));
      end if;
   exception
      when e:others =>
         Mcc.Common_Dialogs.OK_Box("Unable to open: " &
            Filename(1..Filename_Last) & ASCII.LF &
            "Not a valid Robot Scene file?" & ASCII.LF &
            Ada.Exceptions.Exception_Information(E));
   end Save_Scene_Choice;

end Scene_Menu;

