---------------------------------------------------------------
--                                                           
--  ROBOT INTERPRETER
--  By: Martin Carlisle and Keith Shomper     
--                                                           
--  SIMULATION_GUI
--  Description : manually generated GUI objects in main window 
--                       
--  By: Martin Carlisle
--                                         
-- ROBOT INTERPRETER is free software; you can redistribute it and/or 
-- modify it without restriction.  However, we ask that you
-- please retain the original author information, and clearly  
-- indicate if it has been modified.
--  
-- ROBOT INTERPRETER is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
--
---------------------------------------------------------------
with Mcc.Gui.Fonts;
with Main_Window;
with Mcc.Gui.Image;
with Mcc.Gui.Container.Window;
with Mcc.Gui.Widget.Text_Entry;
with Get_Instruction_Window;
with Room_Dialog;
with Ada.Integer_Text_IO;
with RobotADT;
with RoomADT;
with State;
with Instruction_Dialog;
with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;
with Mcc.Gui.Colors;
with Mcc.Gui.Widget.Button.Picture;
with PC_Dialog;
with Controls;
with Mcc.Gui.Widget.Scale;
package body Simulation_Gui is
   function Eight_Bit_Binary(X : in Memory_Value) return String;
   

   Wall          : Mcc.Gui.Image.External_Image;
   Door_North    : Mcc.Gui.Image.External_Image;
   Door_South    : Mcc.Gui.Image.External_Image;
   Door_East     : Mcc.Gui.Image.External_Image;
   Door_West     : Mcc.Gui.Image.External_Image;
   Address_Label : Mcc.Gui.Widget.Label.Label;
   Command_Label : Mcc.Gui.Widget.Label.Label;
   Value_Label   : Mcc.Gui.Widget.Label.Label;
   Decimal_Label : Mcc.Gui.Widget.Label.Label;
   PC_Arrow_Label: Mcc.Gui.Widget.Label.Label;
   
   Courier_10 : Mcc.Gui.Fonts.Font;
   Line_Height : constant := 15;
   Horizontal_Spacing : constant := 5;
   PC_Arrow_Left : constant := 2;
   PC_Arrow_Width : constant := 25;
   Location_Left : constant := PC_Arrow_Left + PC_Arrow_Width;
   Location_Width : constant := 60;
   Memory_Left : constant := Location_Left+Location_Width+Horizontal_Spacing;
   Memory_Width : constant := 80;
   Instruction_Left : constant := Memory_Left+Memory_Width+Horizontal_Spacing;
   Instruction_Width : constant := 110;
   Integer_Left : constant := Instruction_Left+Instruction_Width+Horizontal_Spacing;
   Integer_Width : constant := 45;
   Top_Row : constant := 18;
   
   Grid_Left : constant := Integer_Left + Integer_Width + 5*Horizontal_Spacing;
   Grid_Top : constant := 50;
   Grid_Square_Size : constant := 32;

   procedure Move_Robot(X, Y : in Natural) is
   begin
      Erase_Robot;
      begin
         RobotADT.Set_Location(State.Robot,X,Y,State.Room'access);
      exception when RobotADT.Instruction_Error =>
         Mcc.Gui.Bell;
      end;
      Draw_Robot;
   end Move_Robot;
   
   procedure Toggle_Tile(X, Y : in Natural) is
      use RoomADT;
      Tile : Tile_Type;
      I,J : Natural;
   begin
      -- don't toggle if they've clicked the robot in a doorway
      RobotADT.Get_Location(State.Robot,I,J);
      if I=X and J=Y then
         return;
      end if;
      
      Tile := Get_Tile(State.Room,X,Y);
      if Tile=RoomADT.Wall then
         Set_Tile(State.Room,X,Y,RoomADT.Door);
         if X=0 then
            Set_Image(
               Obj   => Room(X,Room'last(2)-Y),
               Image => Door_West);
         elsif Y=0 then
            Set_Image(
               Obj   => Room(X,Room'last(2)-Y),
               Image => Door_South);
         elsif Y=RoomADT.Size(State.Room)+1 then
            Set_Image(
               Obj   => Room(X,Room'last(2)-Y),
               Image => Door_North);
         else
            Set_Image(
               Obj   => Room(X,Room'last(2)-Y),
               Image => Door_East);
         end if;
      elsif Tile=RoomADT.Door then
         Set_Tile(State.Room,X,Y,RoomADT.Wall);
         Set_Image(
            Obj   => Room(X,Room'last(2)-Y),
            Image => Wall);
      end if;
   end Toggle_Tile;
   
   procedure Room_Click(
      Obj   : in out Mcc.Gui.Sized_Object'Class;
      Event : in     Mcc.Gui.Mouse_Event) is
      use Mcc.Gui;
      X : Integer := Indexed_Picture(Obj).X;
      Y : Integer := Room'Last(2)-Indexed_Picture(Obj).Y;
   begin
      if Event.Button=Left and
         Event.Action=Release  then
         if X <= RoomADT.Size(State.Room)+1 and
            Y <= RoomADT.Size(State.Room)+1 then
            if (X=0 or X=RoomADT.Size(State.Room)+1) then
               if (Y>1 and Y<=RoomADT.Size(State.Room)-1) then
                  Toggle_Tile(X,Y);
               end if;
            elsif (Y=0 or Y=RoomADT.Size(State.Room)+1) then
               if (X>1 and X<=RoomADT.Size(State.Room)-1) then
                  Toggle_Tile(X,Y);
               end if;
            else
               Move_Robot(X,Y);
            end if;
         else
            null;
         end if;
      end if;
   end Room_Click;

   procedure Instruction_Click(
      Obj   : in out Mcc.Gui.Sized_Object'Class;
      Event : in     Mcc.Gui.Mouse_Event) is
      use Mcc.Gui;
      Window_X : Integer := 250;
      Window_Y : Integer := 250;
   begin
      -- am getting clicks on double-clicking in File/Open or Save
      -- dialogs, also only want 1 dialog up at a time
      if State.Dialog_Displayed then
         return;
      end if;
      if Event.Button=Left and
         Event.Action=Release then
         Controls.Stop;
         Instruction_Dialog.Instruction_Y :=
            Indexed_Label(Obj).Y;
         Instruction_Dialog.Instruction_String := To_Unbounded_String(
            Eight_Bit_Binary(
               RobotADT.Get_Address(State.Robot,
                  Instruction_Dialog.Instruction_Y)));
         Get_Instruction_Window.Generate_and_Fill_Window;
         Mcc.Gui.Container.Window.Move(
            Obj => Get_Instruction_Window.Get_Instruction_Window,
            X   => Window_X,
            Y   => Window_Y);
         Mcc.Gui.Widget.Text_Entry.Highlight(
            Get_Instruction_Window.Instruction_Text);
         Mcc.Gui.Set_Key_Listener(
            Obj      => Get_Instruction_Window.Instruction_Text'access,
            Listener => Instruction_Dialog.Key_Callback'access);
      end if;

   end Instruction_Click;

   procedure Location_Click(
      Obj   : in out Mcc.Gui.Sized_Object'Class;
      Event : in     Mcc.Gui.Mouse_Event) is
      use Mcc.Gui;
   begin
      -- am getting clicks on double-clicking in File/Open or Save
      -- dialogs, also only want 1 dialog up at a time
      if State.Dialog_Displayed then
         return;
      end if;
      if Event.Button=Left and
         Event.Action=Release then
         Controls.Stop;
         RobotADT.Set_PC(
            R      => State.Robot,
            State  => Indexed_Label(Obj).Y);
         Erase_Robot;
         Draw_Robot;
      end if;

   end Location_Click;

   function Five_Bit_Binary(X : in Memory_Location) return String is
      Result : String(1..5);
      Copy_X : Integer := X;
   begin
      if Copy_X >= 16 then
         Result(1) := '1';
         Copy_X := Copy_X - 16;
      else
         Result(1) := '0';
      end if;
      if Copy_X >= 8 then
         Result(2) := '1';
         Copy_X := Copy_X - 8;
      else
         Result(2) := '0';
      end if;
      if Copy_X >= 4 then
         Result(3) := '1';
         Copy_X := Copy_X - 4;
      else
         Result(3) := '0';
      end if;
      if Copy_X >= 2 then
         Result(4) := '1';
         Copy_X := Copy_X - 2;
      else
         Result(4) := '0';
      end if;
      if Copy_X = 1 then
         Result(5) := '1';
      else
         Result(5) := '0';
      end if;
      return Result;
   end Five_Bit_Binary;

   function Eight_Bit_Binary(X : in Memory_Value) return String is
      Result : String(1..8);
      Copy_X : Integer := X;
   begin
      if Copy_X >= 128 then
         Result(1) := '1';
         Copy_X := Copy_X - 128;
      else
         Result(1) := '0';
      end if;
      if Copy_X >= 64 then
         Result(2) := '1';
         Copy_X := Copy_X - 64;
      else
         Result(2) := '0';
      end if;
      if Copy_X >= 32 then
         Result(3) := '1';
         Copy_X := Copy_X - 32;
      else
         Result(3) := '0';
      end if;
      Result(4..8) := Five_Bit_Binary(Copy_X);
      return Result;
   end Eight_Bit_Binary;

   
   
   function Integer_Image3(x : in integer) return String is
      result : string(1..3);
   begin
      Ada.Integer_Text_IO.Put(To => Result, Item => x);
      return Result;
   end Integer_Image3;


   procedure Draw_Location(I,J : in Natural) is
      use RoomADT;
   begin
      if get_tile(State.Room,I,J) = RoomADT.wall then
         Set_Image(
            Obj   => Room(I,Room'last(2)-J),
            Image => Wall);
      elsif get_tile(State.Room,I,J) = RoomADT.Door then
         if J=0 then
            Set_Image(
               Obj   => Room(I,Room'last(2)-J),
               Image => Door_South);
         elsif I=0 then
            Set_Image(
               Obj   => Room(I,Room'last(2)-J),
               Image => Door_West);
         elsif J=RoomADT.Size(State.Room)+1 then
            Set_Image(
               Obj   => Room(I,Room'last(2)-J),
               Image => Door_North);
         else
            Set_Image(
               Obj   => Room(I,Room'last(2)-J),
               Image => Door_East);
         end if;
      else
         Set_Image(
            Obj   => Room(I,Room'last(2)-J),
            Image => Blank_Grid);
      end if;
   end Draw_Location;
   
   procedure Erase_Robot is
      I,J              : Natural;
   begin
      RobotADT.Get_Location(State.Robot,I,J);
      Draw_Location(I,J);
   end Erase_Robot;
   
   procedure Draw_Robot is
      use RobotADT;
      I,J              : Natural;
      light_status     : light_type;
      arm_status       : arm_type;
      direction_status : direction_type;
      PC : Integer;
   begin
      RobotADT.Get_Location(State.Robot,I,J);
      J := Room'Last(2)-J;
      RobotADT.Get_Light(State.Robot,light_status);
      RobotADT.Get_Arms(State.Robot,arm_status);
      RobotADT.Get_Direction(State.Robot,direction_status);
      RobotADT.Get_PC(State.Robot,PC);
      Mcc.Gui.Widget.Label.Move(
         Obj => PC_Arrow_Label,
         X   => PC_Arrow_Left,
         Y   => Top_Row+Line_Height*PC);
      Mcc.Gui.Widget.Label.Set_Text(
         Obj  => Main_Window.PC_Value,
         Text => Five_Bit_Binary(PC));
      if light_status=on and arm_status=up and direction_status=north then
         Set_Image(
            Obj   => Room(I,J),
            Image => North_On_Up);
      elsif light_status=on and arm_status=up and direction_status=south then
         Set_Image(
            Obj   => Room(I,J),
            Image => South_On_Up);
      elsif light_status=on and arm_status=up and direction_status=east then
         Set_Image(
            Obj   => Room(I,J),
            Image => East_On_Up);
      elsif light_status=on and arm_status=up and direction_status=west then
         Set_Image(
            Obj   => Room(I,J),
            Image => West_On_Up);
      elsif light_status=on and arm_status=down and direction_status=north then
         Set_Image(
            Obj   => Room(I,J),
            Image => North_On_Down);
      elsif light_status=on and arm_status=down and direction_status=south then
         Set_Image(
            Obj   => Room(I,J),
            Image => South_On_Down);
      elsif light_status=on and arm_status=down and direction_status=east then
         Set_Image(
            Obj   => Room(I,J),
            Image => East_On_Down);
      elsif light_status=on and arm_status=down and direction_status=west then
         Set_Image(
            Obj   => Room(I,J),
            Image => West_On_Down);
      elsif light_status=off and arm_status=up and direction_status=north then
         Set_Image(
            Obj   => Room(I,J),
            Image => North_Off_Up);
      elsif light_status=off and arm_status=up and direction_status=south then
         Set_Image(
            Obj   => Room(I,J),
            Image => South_Off_Up);
      elsif light_status=off and arm_status=up and direction_status=east then
         Set_Image(
            Obj   => Room(I,J),
            Image => East_Off_Up);
      elsif light_status=off and arm_status=up and direction_status=west then
         Set_Image(
            Obj   => Room(I,J),
            Image => West_Off_Up);
      elsif light_status=off and arm_status=down and direction_status=north then
         Set_Image(
            Obj   => Room(I,J),
            Image => North_Off_Down);
      elsif light_status=off and arm_status=down and direction_status=south then
         Set_Image(
            Obj   => Room(I,J),
            Image => South_Off_Down);
      elsif light_status=off and arm_status=down and direction_status=east then
         Set_Image(
            Obj   => Room(I,J),
            Image => East_Off_Down);
      elsif light_status=off and arm_status=down and direction_status=west then
         Set_Image(
            Obj   => Room(I,J),
            Image => West_Off_Down);
      end if;
      if light_status=off then
         Mcc.Gui.Widget.Button.Picture.Set_Image(
            Main_Window.Toggle_Light,
            Simulation_Gui.Light_Off);  
      else
         Mcc.Gui.Widget.Button.Picture.Set_Image(
            Main_Window.Toggle_Light,
            Simulation_Gui.Light_On);  
      end if;
      if arm_status=down then
         Mcc.Gui.Widget.Button.Picture.Set_Image(
            Main_Window.Toggle_Arms,
            North_Off_Down);  
      else
         Mcc.Gui.Widget.Button.Picture.Set_Image(
            Main_Window.Toggle_Arms,
            North_Off_Up);  
      end if;
   end Draw_Robot;
   
   procedure Draw_Room is
      use RoomADT;
   begin
      for I in Room'range(1) loop
         for J in Room'range(2) loop
            Draw_Location(I,J);
         end loop;
      end loop;
      Draw_Robot;
   end Draw_Room;
   
   procedure Display_Instructions is
   begin
      for i in Memory_Labels'range loop
         Set_Text(
            Obj  => Memory_Labels(I),
            Text => Eight_Bit_Binary(
               RobotADT.Get_Address(State.Robot,I)));
         Mcc.Gui.Widget.Label.Set_Text(
            Obj  => Instruction_Labels(I),
            Text => RobotADT.Get_Address(State.Robot,I));
         Mcc.Gui.Widget.Label.Set_Text(
            Obj  => Integer_Labels(I),
            Text => Integer_Image3(RobotADT.Get_Address(State.Robot,I)));
      end loop;
   end Display_Instructions;
   ----------------
   -- Initialize --
   ----------------

   procedure Initialize is
      use type RoomADT.Tile_Type;
      Mapped_J : Integer;
   begin
   
      Mcc.Gui.Fonts.Create(
         Obj    => Courier_10,
         Family => Mcc.Gui.Fonts.Monospaced,
         Size   => 10,
         Style  => Mcc.Gui.Fonts.Plain);

      Mcc.Gui.Widget.Label.Create(
         Obj     => PC_Arrow_Label,
         Parent  => Main_Window.Main_Window,
         X       => PC_Arrow_Left,
         Y       => Top_Row,
         Width   => PC_Arrow_Width,
         Height  => Line_Height,
         Justify => Mcc.Gui.Widget.Label.Left,
         Text    => "=>");
      Mcc.Gui.Fonts.Set_Font(
         Obj      => PC_Arrow_Label,
         New_Font => Courier_10);
      Mcc.Gui.Widget.Label.Set_Foreground_Color(
         Obj       => PC_Arrow_Label,
         New_Color => Mcc.Gui.Colors.Named_Color(Mcc.Gui.Colors.Red));
      Mcc.Gui.Widget.Label.Create(
         Obj     => Address_Label,
         Parent  => Main_Window.Main_Window,
         X       => Location_Left,
         Y       => Top_Row-Line_Height,
         Width   => Location_Width,
         Height  => Line_Height,
         Justify => Mcc.Gui.Widget.Label.Left,
         Text    => "Address");
      Mcc.Gui.Widget.Label.Create(
         Obj     => Address_Label,
         Parent  => Main_Window.Main_Window,
         X       => Memory_Left,
         Y       => Top_Row-Line_Height,
         Width   => Memory_Width,
         Height  => Line_Height,
         Justify => Mcc.Gui.Widget.Label.Left,
         Text    => "Value");
      Mcc.Gui.Widget.Label.Create(
         Obj     => Address_Label,
         Parent  => Main_Window.Main_Window,
         X       => Instruction_Left,
         Y       => Top_Row-Line_Height,
         Width   => Instruction_Width,
         Height  => Line_Height,
         Justify => Mcc.Gui.Widget.Label.Left,
         Text    => "Command");
      Mcc.Gui.Widget.Label.Create(
         Obj     => Address_Label,
         Parent  => Main_Window.Main_Window,
         X       => Integer_Left,
         Y       => Top_Row-Line_Height,
         Width   => 50,
         Height  => Line_Height,
         Justify => Mcc.Gui.Widget.Label.Left,
         Text    => "Decimal");
      for i in Memory_Labels'range loop
         Create(
            Obj     => Location_Labels(i),
            Parent  => Main_Window.Main_Window,
            X       => Location_Left,
            Y       => Top_Row+I*Line_Height,
            Width   => Location_Width,
            Height  => Line_Height,
            Justify => Mcc.Gui.Widget.Label.Left,
            Text    => Five_Bit_Binary(I));
         Location_Labels(I).Y := I;
         Mcc.Gui.Set_Mouse_Listener(
            Obj       => Location_Labels(i)'access,
            Listener  => Location_Click'access);
         Mcc.Gui.Fonts.Set_Font(
            Obj      => Location_Labels(i),
            New_Font => Courier_10);
         Create(
            Obj     => Memory_Labels(i),
            Parent  => Main_Window.Main_Window,
            X       => Memory_Left,
            Y       => Top_Row+I*Line_Height,
            Width   => Memory_Width,
            Height  => Line_Height,
            Justify => Mcc.Gui.Widget.Label.Left,
            Text    => "00000000");
         Memory_Labels(I).Y := I;
         Mcc.Gui.Set_Mouse_Listener(
            Obj       => Memory_Labels(i)'access,
            Listener  => Instruction_Click'access);
         Mcc.Gui.Fonts.Set_Font(
            Obj      => Memory_Labels(i),
            New_Font => Courier_10);
         Mcc.Gui.Widget.Label.Create(
            Obj     => Instruction_Labels(i),
            Parent  => Main_Window.Main_Window,
            X       => Instruction_Left,
            Y       => Top_Row+I*Line_Height,
            Width   => Instruction_Width,
            Height  => Line_Height,
            Justify => Mcc.Gui.Widget.Label.Left,
            Text    => "ADD");
         Mcc.Gui.Fonts.Set_Font(
            Obj      => Instruction_Labels(i),
            New_Font => Courier_10);
         Mcc.Gui.Widget.Label.Create(
            Obj     => Integer_Labels(i),
            Parent  => Main_Window.Main_Window,
            X       => Integer_Left,
            Y       => Top_Row+I*Line_Height,
            Width   => Integer_Width,
            Height  => Line_Height,
            Justify => Mcc.Gui.Widget.Label.Right,
            Text    => "0");
         Mcc.Gui.Fonts.Set_Font(
            Obj      => Integer_Labels(i),
            New_Font => Courier_10);
      end loop;

      Mcc.Gui.Image.Create(
         Obj  => Blank_Grid,
         Name => "blank_grid.gif");
      Mcc.Gui.Image.Create(
         Obj  => Wall,
         Name => "wall.gif");
      Mcc.Gui.Image.Create(
         Obj  => North_On_Up,
         Name => "north_up_on.gif");
      Mcc.Gui.Image.Create(
         Obj  => North_On_Down,
         Name => "north_down_on.gif");
      Mcc.Gui.Image.Create(
         Obj  => North_Off_Up,
         Name => "north_up_off.gif");
      Mcc.Gui.Image.Create(
         Obj  => North_Off_Down,
         Name => "north_down_off.gif");
      Mcc.Gui.Image.Create(
         Obj  => South_On_Up,
         Name => "south_up_on.gif");
      Mcc.Gui.Image.Create(
         Obj  => South_On_Down,
         Name => "south_down_on.gif");
      Mcc.Gui.Image.Create(
         Obj  => South_Off_Up,
         Name => "south_up_off.gif");
      Mcc.Gui.Image.Create(
         Obj  => South_Off_Down,
         Name => "south_down_off.gif");
      Mcc.Gui.Image.Create(
         Obj  => East_On_Up,
         Name => "east_up_on.gif");
      Mcc.Gui.Image.Create(
         Obj  => East_On_Down,
         Name => "east_down_on.gif");
      Mcc.Gui.Image.Create(
         Obj  => East_Off_Up,
         Name => "east_up_off.gif");
      Mcc.Gui.Image.Create(
         Obj  => East_Off_Down,
         Name => "east_down_off.gif");
      Mcc.Gui.Image.Create(
         Obj  => West_On_Up,
         Name => "west_up_on.gif");
      Mcc.Gui.Image.Create(
         Obj  => West_On_Down,
         Name => "west_down_on.gif");
      Mcc.Gui.Image.Create(
         Obj  => West_Off_Up,
         Name => "west_up_off.gif");
      Mcc.Gui.Image.Create(
         Obj  => West_Off_Down,
         Name => "west_down_off.gif");
      Mcc.Gui.Image.Create(
         Obj  => Light_On,
         Name => "light_on.gif");
      Mcc.Gui.Image.Create(
         Obj  => Light_Off,
         Name => "light_off.gif");
      Mcc.Gui.Image.Create(
         Obj  => Door_North,
         Name => "door_north.gif");
      Mcc.Gui.Image.Create(
         Obj  => Door_South,
         Name => "door_south.gif");
      Mcc.Gui.Image.Create(
         Obj  => Door_East,
         Name => "door_east.gif");
      Mcc.Gui.Image.Create(
         Obj  => Door_West,
         Name => "door_west.gif");
      Mcc.Gui.Image.Create(
         Obj  => Wavy_EW,
         Name => "wavy_ew.gif");
      Mcc.Gui.Image.Create(
         Obj  => Wavy_NS,
         Name => "wavy_ns.gif");
      for I in Room'range(1) loop
         for J in Room'range(2) loop
            Mapped_J := Room'Last(2)-J;
            if RoomADT.get_tile(State.Room,I,J) = RoomADT.wall then
               Create(
                  Obj    => Room(I,Mapped_J),
                  Parent => Main_Window.Main_Window,
                  X      => Grid_Left+(I-Room'First(1))*Grid_Square_Size,
                  Y      => Grid_Top+(Mapped_J-Room'First(2))*Grid_Square_Size,
                  Width  => Grid_Square_Size,
                  Height => Grid_Square_Size,
                  Image  => Wall);
            elsif RoomADT.get_tile(State.Room,I,J) = RoomADT.Door then
               if I=0 then
                  Create(
                     Obj    => Room(I,Mapped_J),
                     Parent => Main_Window.Main_Window,
                     X      => Grid_Left+(I-Room'First(1))*Grid_Square_Size,
                     Y      => Grid_Top+(Mapped_J-Room'First(2))*Grid_Square_Size,
                     Width  => Grid_Square_Size,
                     Height => Grid_Square_Size,
                     Image  => Door_West);
               elsif J=0 then
                  Create(
                     Obj    => Room(I,Mapped_J),
                     Parent => Main_Window.Main_Window,
                     X      => Grid_Left+(I-Room'First(1))*Grid_Square_Size,
                     Y      => Grid_Top+(Mapped_J-Room'First(2))*Grid_Square_Size,
                     Width  => Grid_Square_Size,
                     Height => Grid_Square_Size,
                     Image  => Door_South);
               elsif J=RoomADT.Size(State.Room)+1 then
                  Create(
                     Obj    => Room(I,Mapped_J),
                     Parent => Main_Window.Main_Window,
                     X      => Grid_Left+(I-Room'First(1))*Grid_Square_Size,
                     Y      => Grid_Top+(Mapped_J-Room'First(2))*Grid_Square_Size,
                     Width  => Grid_Square_Size,
                     Height => Grid_Square_Size,
                     Image  => Door_North);
               else
                  Create(
                     Obj    => Room(I,Mapped_J),
                     Parent => Main_Window.Main_Window,
                     X      => Grid_Left+(I-Room'First(1))*Grid_Square_Size,
                     Y      => Grid_Top+(Mapped_J-Room'First(2))*Grid_Square_Size,
                     Width  => Grid_Square_Size,
                     Height => Grid_Square_Size,
                     Image  => Door_East);
               end if;
            else
               Create(
                  Obj    => Room(I,Mapped_J),
                  Parent => Main_Window.Main_Window,
                  X      => Grid_Left+(I-Room'First(1))*Grid_Square_Size,
                  Y      => Grid_Top+(Mapped_J-Room'First(2))*Grid_Square_Size,
                  Width  => Grid_Square_Size,
                  Height => Grid_Square_Size,
                  Image  => Blank_Grid);
            end if;
            Room(I,J).X := I;
            Room(I,J).Y := J;
            Mcc.Gui.Set_Mouse_Listener(
               Obj       => Room(I,Mapped_J)'access,
               Listener  => Room_Click'access);
         end loop;
      end loop;
      Mcc.Gui.Set_Mouse_Listener(
         Obj       => Main_Window.PC_Value'access,
         Listener  => PC_Dialog.PC_Click'access);
      RobotADT.Set_Location(
         R    => State.Robot,
         X    => 1,
         Y    => 1,
         Room => State.Room'access);
      Mcc.Gui.Widget.Scale.Set(
         Obj      => Main_Window.Speed_Scale,
         Location => 50);
      Draw_Robot;
      Display_Instructions;
   end Initialize;

   ---------------------
   -- Set_Instruction --
   ---------------------

   procedure Set_Instruction (Which : in Memory_Location) is
   begin
      Set_Text(
         Obj  => Memory_Labels(Which),
         Text => Eight_Bit_Binary(
            RobotADT.Get_Address(State.Robot,Which)));
      Mcc.Gui.Widget.Label.Set_Text(
         Obj  => Instruction_Labels(Which),
         Text => RobotADT.Get_Address(State.Robot,Which));
      Mcc.Gui.Widget.Label.Set_Text(
         Obj  => Integer_Labels(Which),
         Text => Integer_Image3(RobotADT.Get_Address(State.Robot,Which)));
   end Set_Instruction;

   -------------------
   -- Set_Room_Size --
   -------------------

   procedure Set_Room_Size (Size : in Room_Size_Type; Number_Doors : Natural) is
      I,J : Integer;
   begin
      RobotADT.Get_Location(State.Robot,I,J);
      RoomADT.Create(R => State.Room, Size => Size, Num_Doors => Number_Doors);
      if I>=Size+1 or J>=Size+1 then
         Move_Robot(1,1);
      end if;
      Draw_Room;
   end Set_Room_Size;

end Simulation_Gui;

